<?php

namespace Drupal\asf\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\asf\AsfSchema;

/**
 * Plugin implementation of the 'AsfWidget' widget.
 *
 * @FieldWidget(
 *   id = "AsfWidget",
 *   module = "asf",
 *   label = @Translation("Asf publication settings"),
 *   field_types = {
 *     "asf"
 *   }
 * )
 */
class AsfWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {


    $field_settings = $this->getFieldSettings();
    $widget = array();
    $publication_type = isset($items[$delta]->publication_type) ? $items[$delta]->publication_type : ASF_TYPE_NOTHING;

    if (!empty($items[$delta]->startdate)) {
      $startdate = isset($items[$delta]->startdate) ? date('Y-m-d H:i', $items[$delta]->startdate) : date('Y-m-d H:i', REQUEST_TIME);
    }
    else {
      $startdate = '';
    }

    if (!empty($items[$delta]->enddate)) {
      $enddate = isset($items[$delta]->enddate) ? date('Y-m-d H:i', $items[$delta]->enddate) : '';
    }
    else {
      $enddate = '';
    }

    $options = array(
      ASF_TYPE_NOTHING => 'Do nothing.',
      ASF_TYPE_START => 'publish from a DATE.',
      ASF_TYPE_START_END => 'publish on DATE, unpublish on end DATE',
      ASF_TYPE_ITERATE => 'publish from start TIME to end TIME, repeat multiple days',
      ASF_TYPE_ITERATE_AMOUNT => 'publish from start TIME to end TIME, repeat x times',
    );
    $widget['publication_type'] = array(
      '#type' => 'select',
      '#title' => t('How do you want this entity to be published.'),
      '#default_value' => $publication_type,
      '#options' => $options,
      '#attributes' => array('class' => array('iteration_toggler')),
      '#attached' => array(
        'library' => array(
          'asf/asf.field',
        ),
      ),
    );

//    $widget['iteration_end'] = array(
//      '#type' => 'select',
//      '#title' => t('Type of iteration'),
//      '#default_value' => isset($items[$delta]->iteration_end) ? $items[$delta]->iteration_end : 0,
//      '#attributes' => array(
//        'class' => array(
//          'interation_end',
//          ASF_TYPE_ITERATE
//        )
//      ),
//      '#options' => array(
//        ASF_ITERATION_ENDDATE => t('Loop until end date'),
//        ASF_ITERATION_MAX => t('Loop until amount of iterations reached'),
//        ASF_ITERATION_FIRST => t('Loop until first of both reached'),
//        ASF_ITERATION_INFINITE => t('Loop until the end of times'),
//      ),
//    );
    $widget['startdate'] = array(
      '#type' => 'textfield',
      '#title' => t('Start date/time of the publishing.'),
      '#default_value' => $startdate,
      //'#element_validate' => array('_asf_field_validate_ISO_date'),
      '#attributes' => array(
        'class' => array(
          'timepicker',
          ASF_TYPE_START,
          ASF_TYPE_START_END,
          ASF_TYPE_ITERATE,
          ASF_TYPE_ITERATE_AMOUNT,
        ),
      ),
    );
    $widget['enddate'] = array(
      '#type' => 'textfield',
      '#title' => t('End date/time of the publishing.'),
      '#default_value' => $enddate,
      '#attributes' => array(
        'class' => array(
          'timepicker',
          ASF_TYPE_START_END,
          ASF_TYPE_ITERATE,
        ),
      ),
    );
    $widget['start_time'] = array(
      '#type' => 'textfield',
      '#title' => t('Publish at'),
      '#description' => t('in 24 h format e.g: 22:34'),
      '#default_value' => isset($items[$delta]->start_time) ? $items[$delta]->start_time : '',

      '#attributes' => array(
        'class' => array(
          'timepicker_time',
          ASF_TYPE_ITERATE,
          ASF_TYPE_ITERATE_AMOUNT,
          'asf-inline',
        ),
      ),
      '#size' => 6,
    );
    $widget['end_time'] = array(
      '#type' => 'textfield',
      '#title' => t('unpublish at'),
      '#description' => t('in 24 h format e.g: 22:34'),
      '#default_value' => isset($items[$delta]->end_time) ? $items[$delta]->end_time : '',
      '#attributes' => array(
        'class' => array(
          'timepicker_time',
          ASF_TYPE_ITERATE,
          ASF_TYPE_ITERATE_AMOUNT,
          'asf-inline',
        ),
      ),
      '#size' => 6,
    );

    $widget['iteration_day'] = array(
      '#type' => 'textfield',
      '#title' => t('Iteration day'),
      '#default_value' => isset($items[$delta]->iteration_day) ? $items[$delta]->iteration_day : '*',
      '#size' => 5,
      //'#element_validate' => array('_asf_field_validate_iteration_integer_list'),
      '#attributes' => array(
        'class' => array(
          ASF_TYPE_ITERATE,
          ASF_TYPE_ITERATE_AMOUNT,
          'asf-inline',
          'asf-clear',
        ),
      ),
    );
    $widget['iteration_week'] = array(
      '#type' => 'textfield',
      '#title' => t('Iteration week'),
      '#default_value' => isset($items[$delta]->iteration_week) ? $items[$delta]->iteration_week : '*',
      '#size' => 5,
      '#attributes' => array(
        'class' => array(
          ASF_TYPE_ITERATE,
          ASF_TYPE_ITERATE_AMOUNT,
          'asf-inline',
        ),
      ),
    );
    $widget['iteration_weekday'] = array(
      '#type' => 'textfield',
      '#title' => t('Iteration weekday'),
      '#default_value' => isset($items[$delta]->iteration_weekday) ? $items[$delta]->iteration_weekday : '*',
      '#size' => 5,
      //'#element_validate' => array('_asf_field_validate_iteration_integer_list'),
      '#attributes' => array(
        'class' => array(
          ASF_TYPE_ITERATE,
          ASF_TYPE_ITERATE_AMOUNT,
          'asf-inline',
        ),
      ),
    );

    $widget['iteration_month'] = array(
      '#type' => 'textfield',
      '#title' => t('Iteration month'),
      '#default_value' => isset($items[$delta]->iteration_month) ? $items[$delta]->iteration_month : '*',
      '#size' => 5,
      //'#element_validate' => array('_asf_field_validate_iteration_integer_list'),
      '#attributes' => array(
        'class' => array(
          ASF_TYPE_ITERATE,
          ASF_TYPE_ITERATE_AMOUNT,
          'asf-inline',
        ),
      ),
    );

    $widget['iteration_year'] = array(
      '#type' => 'textfield',
      '#title' => t('Iteration year'),
      '#default_value' => isset($items[$delta]->iteration_year) ? $items[$delta]->iteration_year : '*',
      '#size' => 5,
      //'#element_validate' => array('_asf_field_validate_iteration_integer_list'),
      '#attributes' => array(
        'class' => array(
          ASF_TYPE_ITERATE,
          ASF_TYPE_ITERATE_AMOUNT,
          'asf-inline',
        ),
      ),
    );

    $widget['iteration_max'] = array(
      '#type' => 'textfield',
      '#title' => t('Iteration max'),
      '#description' => t('The iteration will occurr this many times'),
      '#default_value' => isset($items[$delta]->iteration_max) ? $items[$delta]->iteration_max : '',
      '#size' => 5,
      '#attributes' => array(
        'class' => array(
          //ASF_TYPE_ITERATE,
          ASF_TYPE_ITERATE_AMOUNT,
          'asf-inline',
        ),
      ),
    );


    $widget['inherit_eid'] = array(
      '#type' => 'textfield',
      '#title' => t('inherit Eid'),
      '#default_value' => isset($items[$delta]->inherit_eid) ? $items[$delta]->inherit_eid : '',
      '#attributes' => array(
        'class' => array(
          ASF_TYPE_INHERIT,
          'asf-inline',
        )
      ),
    );

    return $widget;
  }

  /**
   * Validate the color text field.
   */
  public function validate($element, FormStateInterface $form_state) {
    $value = $element['#value'];
//    if (strlen($value) == 0) {
//      $form_state->setValueForElement($element, '');
//      return;
//    }
//    if (!preg_match('/^#([a-f0-9]{6})$/iD', strtolower($value))) {
//      $form_state->setError($element, t("Color must be a 6-digit hexadecimal value, suitable for CSS."));
//    }
  }

}
